/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/config/core/v3/socket_option.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "envoy/config/core/v3/socket_option.upb.h"
#include "udpa/annotations/status.upb.h"
#include "udpa/annotations/versioning.upb.h"
#include "validate/validate.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout_field envoy_config_core_v3_SocketOption__fields[6] = {
  {1, UPB_SIZE(24, 24), 0, 0, 9, 1},
  {2, UPB_SIZE(0, 0), 0, 0, 3, 1},
  {3, UPB_SIZE(8, 8), 0, 0, 3, 1},
  {4, UPB_SIZE(32, 40), UPB_SIZE(-41, -57), 0, 3, 1},
  {5, UPB_SIZE(32, 40), UPB_SIZE(-41, -57), 0, 12, 1},
  {6, UPB_SIZE(16, 16), 0, 0, 14, 1},
};

const upb_msglayout envoy_config_core_v3_SocketOption_msginit = {
  NULL,
  &envoy_config_core_v3_SocketOption__fields[0],
  UPB_SIZE(48, 64), 6, false,
};

#include "upb/port_undef.inc"

