/* This file was generated by upbc (the upb compiler) from the input
 * file:
 *
 *     envoy/annotations/resource.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated. */

#include <stddef.h>
#include "upb/msg.h"
#include "envoy/annotations/resource.upb.h"
#include "google/protobuf/descriptor.upb.h"

#include "upb/port_def.inc"

static const upb_msglayout_field envoy_annotations_ResourceAnnotation__fields[1] = {
  {1, UPB_SIZE(0, 0), 0, 0, 9, 1},
};

const upb_msglayout envoy_annotations_ResourceAnnotation_msginit = {
  NULL,
  &envoy_annotations_ResourceAnnotation__fields[0],
  UPB_SIZE(8, 16), 1, false,
};

#include "upb/port_undef.inc"

